package server.network;

import java.security.PrivateKey;
import java.security.SecureRandom;
import java.util.Arrays;

import javax.crypto.SecretKey;

import common.color.TextColor;
import common.init.Config;
import common.log.Log;
import common.network.ILoginHandler;
import common.network.IPlayer;
import common.network.NetConnection;
import common.network.NetHandler;
import common.packet.LPacketPasswordResponse;
import common.packet.LPacketStartEncrypt;
import common.packet.RPacketDisconnect;
import common.packet.RPacketRequestEncrypt;
import server.Server;

public class LoginHandler extends NetHandler implements ILoginHandler
{
    private static enum LoginState {
        INIT, ENCRYPT, PASSWORD, AUTHENTICATED, ACCEPTED;
    }
    
    private static final SecureRandom TOKEN_RNG = new SecureRandom();
    
    private final Server server;
    public final NetConnection netManager;
    
    private LoginState state = LoginState.INIT;
    private int timer;
    private String loginUser;
    private String loginPass;
    private byte[] loginToken;

    public LoginHandler(Server server, NetConnection netManager)
    {
        this.netManager = netManager;
        this.server = server;
    }

    public void closeConnection(String reason)
    {
        try
        {
            Log.JNI.info("Trenne " + this.getConnectionInfo());
            this.netManager.sendPacket(new RPacketDisconnect(reason));
            this.netManager.closeChannel(reason);
        }
        catch (Exception exception)
        {
            Log.JNI.error(exception, "Fehler beim Trennen des Spielers beim Login");
        }
    }
    
    public void onDisconnect(String reason)
    {
        Log.JNI.info(this.getConnectionInfo() + " wurde beim Login getrennt: " + TextColor.stripCodes(reason));
    }

    public String getConnectionInfo()
    {
        return this.loginUser != null ? (this.loginUser + " (" + this.netManager.getCutAddress()
        		+ ")") : this.netManager.getCutAddress();
    }
    
    public void update()
    {
        if(this.state == LoginState.AUTHENTICATED)
        	this.tryAcceptPlayer();
//        else if (this.currentLoginState == LoginState.DELAY_ACCEPT)
//        {
//            if (this.server.getPlayer(this.loginUser) == null)
//            {
//                this.currentLoginState = LoginState.ACCEPTED;
//                this.server.addPlayer(this.networkManager, this.loginUser);
//            }
//        }
        if(this.timer++ == 600)
            this.closeConnection("Anmeldung dauerte zu lange");
    }

    private void tryAcceptPlayer()
    {
		if(this.server.getPlayer(this.loginUser) != null) {
			this.closeConnection("Nutzername '" + this.loginUser + "' ist bereits vergeben.");
			return;
		}
//            if (this.server.getPlayer(this.loginUser) != null)
//            {
//            	this.closeConnection("Nutzername '" + this.loginUser + "' ist bereits vergeben.");
//            	return;
////                this.currentLoginState = LoginState.DELAY_ACCEPT;
////                player.netHandler.kick("Du hast dich von einen anderen Ort verbunden");
//            }
//            this.networkManager.sendPacket(new RPacketLoginSuccess());
        String kick = this.server.addPlayer(this.netManager, this.loginUser, this.loginPass);
        if(kick != null)
        	this.closeConnection(kick);
        else
            this.state = LoginState.ACCEPTED;
    }

    public void sendLoginPacket() {
		if(this.state != LoginState.INIT)
			throw new IllegalStateException("Unerwartetes Handshake-Paket");
		this.state = LoginState.ENCRYPT;
		this.loginToken = new byte[4];
		TOKEN_RNG.nextBytes(this.loginToken);
    	this.netManager.sendPacket(new RPacketRequestEncrypt(this.server.getPublicKey(), this.loginToken));
    }
    
	public void processEncryption(LPacketStartEncrypt packet) {
		if(this.state != LoginState.ENCRYPT)
			throw new IllegalStateException("Unerwartetes Verschlüsselungs-Paket");
        PrivateKey pkey = this.server.getPrivateKey();
        if(!Arrays.equals(this.loginToken, packet.getToken(pkey)))
            throw new IllegalStateException("Fehlerhaftes Token");
        SecretKey key = packet.getKey(pkey);
        this.netManager.startEncryption(key);
		this.state = LoginState.PASSWORD;
	}
    
    public void processPasswordResponse(LPacketPasswordResponse packetIn) {
		if(this.state != LoginState.PASSWORD)
			throw new IllegalStateException("Unerwartetes Passwort-Paket");
		this.loginUser = packetIn.getUser();
		this.loginPass = packetIn.getPassword();
        if(this.loginUser.isEmpty() || !IPlayer.isValidUser(this.loginUser))
        	throw new IllegalStateException("Ungültiger Nutzername!");
//    		if(!this.checkConnect(packetIn.getAccess()))
//    			return;
    	if(!Config.password.isEmpty() && !Config.password.equals(packetIn.getAccess())) {
    		this.closeConnection("Falsches Zugangspasswort");
    		return;
		}
		this.state = LoginState.AUTHENTICATED;
	}
}
