package server.world;

import java.util.Map;
import java.util.Set;
import common.biome.Biome;
import common.block.Block;
import common.entity.Entity;
import common.init.BlockRegistry;
import common.init.Blocks;
import common.log.Log;
import common.rng.Random;
import common.tileentity.TileEntity;
import common.util.BlockPos;
import common.world.BlockArray;
import common.world.Chunk;
import common.world.State;
import common.world.World;
import server.worldgen.BiomeGenerator;

public class ChunkServer extends Chunk {
	private long lastSave;
	private long inhabited;

	public ChunkServer(World world, int x, int z) {
		super(world, x, z);
	}

	public ChunkServer(World world, short[] data, int height, boolean base, boolean ceil, Random rand, Biome[] biomes, int x, int z) {
		this(world, x, z);
		boolean sky = !world.dimension.hasNoLight();
		for(int bx = 0; bx < 16; ++bx) {
			for(int bz = 0; bz < 16; ++bz) {
				for(int by = 0; by < height; ++by) {
					State state = BlockRegistry.STATEMAP.getByValue(data[bx << 4 | bz | by << 8]);
					if(state != null && state.getBlock() != Blocks.air) {
						int y = by >> 4;
						BlockArray arr = this.getArray(y);
						if(arr == null) {
							arr = new BlockArray(y << 4, sky, null);
							this.setArray(arr);
						}
						arr.set(bx, by & 15, bz, state);
					}
				}
			}
		}
		if(base) {
			Block caves = world.dimension.getCaveFiller().getBlock();
			BlockArray arr = this.getArray(0);
			if(arr == null) {
				arr = new BlockArray(0, sky, null);
				this.setArray(arr);
			}
			for(int bx = 0; bx < 16; ++bx) {
				for(int bz = 0; bz < 16; ++bz) {
					for(int by = 0; by < 5; ++by) {
						if(by <= rand.zrange(5)) {
							Block block = arr.get(bx, by, bz).getBlock();
							if(block == Blocks.air || block.getMaterial().isLiquid() || block == caves)
								arr.set(bx, by, bz, this.filler);
						}
					}
				}
			}
		}
		if(ceil) {
			int y = (height - 1) >> 4;
			BlockArray arr = this.getArray(y);
			if(arr == null) {
				arr = new BlockArray(y << 4, sky, null);
				this.setArray(arr);
			}
			y = (height - 5) >> 4;
			arr = this.getArray(y);
			if(arr == null) {
				arr = new BlockArray(y << 4, sky, null);
				this.setArray(arr);
			}
			for(int bx = 0; bx < 16; ++bx) {
				for(int bz = 0; bz < 16; ++bz) {
					for(int by = height - 1; by >= height - 5; --by) {
						if(by >= (height - 1) - rand.zrange(5))
							this.getArray(by >> 4).set(bx, by & 15, bz, Blocks.air.getState());
					}
				}
			}
		}
		for(int n = 0; n < this.biomes.length; ++n) {
			this.biomes[n] = (byte)biomes[n].id;
		}
		if(ceil)
			this.resetRelight();
		else
			this.genSkyLight();
	}
	
	public int getTopSegment() {
		return this.top;
	}

	public int getBottomSegment() {
		return this.bottom;
	}

	public Set<BlockArray> getStorage() {
		return this.blockList;
	}
	
	public void addTileEntity(TileEntity tile) {
		this.addTileEntity(tile.getPos(), tile);

		if(this.loaded) {
			this.world.addTileEntity(tile);
		}
	}

	public void onChunkLoad() {
		this.loaded = true;
		this.world.addTileEntities(this.tiles.values());

		for(int n = 0; n < this.entities.length; ++n) {
			for(Entity entity : this.entities[n]) {
				entity.onChunkLoad();
			}

			this.world.loadEntities(this.entities[n]);
		}
	}
	
	public boolean isDirty(long time) {
		if(this.hasEntity && time != this.lastSave || this.modified) {
			return true;
		}
		return this.modified;
	}
	
	public void setStorage(BlockArray[] data) {
		for(BlockArray arr : data) {
			this.setArray(arr);
		}
	}

	public Biome getBiome(BlockPos pos, BiomeGenerator gen) {
		int x = pos.getX() & 15;
		int z = pos.getZ() & 15;
		int o = this.biomes[z << 4 | x] & 255;

		if(o == 255) {
			Biome biome = gen == null ? Biome.DEF_BIOME : gen.getBiomeGenerator(pos, Biome.DEF_BIOME);
			o = biome.id;
			this.biomes[z << 4 | x] = (byte)(o & 255);
		}

		return Biome.getBiomeDef(o);
	}

	public byte[] getBiomes() {
		return this.biomes;
	}

	public void setBiomes(byte[] biomes) {
		if(this.biomes.length != biomes.length) {
			Log.JNI.warn("Konnte Biome des Chunks nicht setzen, Länge des Arrays ist " + biomes.length + " statt " + this.biomes.length);
		}
		else {
			for(int n = 0; n < this.biomes.length; ++n) {
				this.biomes[n] = biomes[n];
			}
		}
	}

	public int[] getHeights() {
		return this.height;
	}

	public void setHeights(int[] map) {
		if(this.height.length != map.length) {
			Log.JNI.warn("Konnte Höhen des Chunks nicht setzen, Länge des Arrays ist " + map.length + " statt " + this.height.length);
		}
		else {
			for(int n = 0; n < this.height.length; ++n) {
				this.height[n] = map[n];
			}
		}
	}

	public Map<BlockPos, TileEntity> getTiles() {
		return this.tiles;
	}

	public boolean isTerrainPopulated() {
		return this.populated;
	}

	public void setTerrainPopulated(boolean populated) {
		this.populated = populated;
	}

	public boolean isLightPopulated() {
		return this.lightInit;
	}

	public void setLightPopulated(boolean populated) {
		this.lightInit = populated;
	}

	public void setModified(boolean modified) {
		this.modified = modified;
	}

	public void setHasEntities(boolean entities) {
		this.hasEntity = entities;
	}

	public void setSaved(long time) {
		this.lastSave = time;
	}

	public long getInhabited() {
		return this.inhabited;
	}

	public void setInhabited(long time) {
		this.inhabited = time;
	}
}
